package net.wanji.opt.service.signalopt.impl;

import net.wanji.databus.dao.mapper.CrossDataHistMapper;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.opt.service.signalopt.GreenBeltInfoService;
import net.wanji.opt.vo.GreenBeltFlowStopTimeVO;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/12/02 13:42
 */
@Service
public class GreenBeltServiceImpl implements GreenBeltInfoService {

    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;
    @Resource
    private CrossDataHistMapper crossDataHistMapper;

    @Override
    public List<GreenBeltFlowStopTimeVO> greenBeltCrossDetailHist(Integer greenId) {
        List<String> crossIds = greenwaveInfoMapper.selectCrossIdsById(greenId);
        long currentTimeMillis = System.currentTimeMillis();
        int startSecond = (int) (currentTimeMillis / 1000 - 3600);
        int endSecond = (int) (currentTimeMillis / 1000);
        List<CrossDataHistPO> crossDataHistPOS = crossDataHistMapper.selectByCrossIdsAndTimestamp(crossIds, startSecond , endSecond);
        if (!CollectionUtils.isEmpty(crossDataHistPOS)) {
            List<GreenBeltFlowStopTimeVO> results = new ArrayList<>();
            Map<Date, List<CrossDataHistPO>> startTimeMap = crossDataHistPOS.stream().collect(Collectors.groupingBy(CrossDataHistPO::getStartTime));
            for (Map.Entry<Date, List<CrossDataHistPO>> entry : startTimeMap.entrySet()) {
                GreenBeltFlowStopTimeVO greenBeltFlowStopTimeVO = new GreenBeltFlowStopTimeVO();
                Date startDate = entry.getKey();
                List<GreenBeltFlowStopTimeVO.FlowStopTimeDetail> crossList = new ArrayList<>();
                greenBeltFlowStopTimeVO.setStartTime(startDate);
                List<CrossDataHistPO> value = entry.getValue();
                if (!CollectionUtils.isEmpty(value)) {
                    for (CrossDataHistPO crossDataHistPO : value) {
                        GreenBeltFlowStopTimeVO.FlowStopTimeDetail flowStopTimeDetail = new GreenBeltFlowStopTimeVO.FlowStopTimeDetail();
                        flowStopTimeDetail.setFlow(crossDataHistPO.getFlow());
                        flowStopTimeDetail.setStopTimes(crossDataHistPO.getStopTimes());
                        flowStopTimeDetail.setCrossId(crossDataHistPO.getCrossId());
                        // todo 先删除，后续通过路口距离/速度计算 距离通过卫博发送数据
                        flowStopTimeDetail.setTravelTime(0.0);
                        crossList.add(flowStopTimeDetail);
                    }
                }
                greenBeltFlowStopTimeVO.setDetailList(crossList);
                results.add(greenBeltFlowStopTimeVO);
            }
            Collections.sort(results, Comparator.comparing(GreenBeltFlowStopTimeVO::getStartTime));
            return results;
        }
        return Collections.EMPTY_LIST;
    }
}
